/* $Id: dexter-channel-internal.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTER_CHANNEL_INTERNAL_H__
#define __DEXTER_CHANNEL_INTERNAL_H__

#include <glib-object.h>
#include <dexter-channel.h>
#include <dexter-channel-object.h>

G_BEGIN_DECLS

#define DEFAULT_CHANNEL_HOST                "127.0.0.1"
#define DEFAULT_CHANNEL_SERVICE             "3663"
#define DEFAULT_CHANNEL_SOCKET_OPTIONS      NULL
#define DEFAULT_CHANNEL_RECV_BUFFER_SIZE    4096
#define DEFAULT_CHANNEL_TIMEOUT_MSECS       30000 /* 30,000 millisecs = 30 seconds */
#define DEFAULT_CHANNEL_TLS_CERT_FILE       "/etc/ssl/libdexter/channel-cert.pem"
#define DEFAULT_CHANNEL_TLS_KEY_FILE        "/etc/ssl/libdexter/channel-key.pem"
#define DEFAULT_CHANNEL_TLS_TRUST_FILE      "/etc/ssl/libdexter/channel-trust.pem"
#define DEFAULT_CHANNEL_TLS_CRL_FILE        "/etc/ssl/libdexter/channel-crl.pem"
#define DEFAULT_CHANNEL_TLS_VERIFY_HOSTNAME TRUE

#define DEXTER_TYPE_CHANNEL             (dexter_channel_get_type ())
#define DEXTER_CHANNEL(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj), \
                                            DEXTER_TYPE_CHANNEL, DexterChannel))
#define DEXTER_CHANNEL_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), \
                                            DEXTER_TYPE_CHANNEL, DexterChannelClass))
#define DEXTER_IS_CHANNEL(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
                                            DEXTER_TYPE_CHANNEL))
#define DEXTER_IS_CHANNEL_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), \
                                            DEXTER_TYPE_CHANNEL))
#define DEXTER_CHANNEL_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), \
                                            DEXTER_TYPE_CHANNEL, DexterChannelClass))

typedef struct _DexterChannelClass DexterChannelClass;

GType     dexter_channel_get_type                   (void) G_GNUC_INTERNAL G_GNUC_CONST;

const DexterChannelSettings
         *dexter_channel_get_settings               (DexterChannel       *channel) G_GNUC_INTERNAL;

gboolean  dexter_channel_register_channel_object    (DexterChannel       *channel,
                                                     DexterChannelObject *channel_object,
                                                     gint                 wait_types) G_GNUC_INTERNAL;

gboolean  dexter_channel_deregister_channel_object  (DexterChannel       *channel,
                                                     DexterChannelObject *channel_object) G_GNUC_INTERNAL;

void      dexter_channel_command_push               (DexterChannel       *channel,
                                                     DexterChannelObject *channel_object,
                                                     gint                 command,
                                                     gint                 parameter,
                                                     GString             *payload,
                                                     GError             **error) G_GNUC_INTERNAL;
G_END_DECLS

#endif /* __DEXTER_CHANNEL_INTERNAL_H__ */
