/* $Id: dexter-channel-object.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTER_CHANNEL_OBJECT_H__
#define __DEXTER_CHANNEL_OBJECT_H__

#include <glib-object.h>
#include <dexter-channel.h>

G_BEGIN_DECLS

#define DEFAULT_CHANNEL_OBJECT_TIMEOUT_MSECS 30000 /* 30,000 millisecs = 30 seconds */

enum DexterChannelMessageType  
{ 
  DEXTER_MESSAGE_COMMAND = 1<<0, 
  DEXTER_MESSAGE_REPLY   = 1<<1,
  DEXTER_MESSAGE_DATA    = 1<<2
};

typedef struct _DexterChannelMessage DexterChannelMessage;

#define DEXTER_TYPE_CHANNEL_OBJECT              (dexter_channel_object_get_type ())
#define DEXTER_CHANNEL_OBJECT(obj)              (G_TYPE_CHECK_INSTANCE_CAST((obj), \
                                                    DEXTER_TYPE_CHANNEL_OBJECT, DexterChannelObject))
#define DEXTER_CHANNEL_OBJECT_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), \
                                                    DEXTER_TYPE_CHANNEL_OBJECT, DexterChannelObjectClass))
#define DEXTER_IS_CHANNEL_OBJECT(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
                                                    DEXTER_TYPE_CHANNEL_OBJECT))
#define DEXTER_IS_CHANNEL_OBJECT_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), \
                                                    DEXTER_TYPE_CHANNEL_OBJECT))
#define DEXTER_CHANNEL_OBJECT_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), \
                                                    DEXTER_TYPE_CHANNEL_OBJECT, DexterChannelObjectClass))

typedef struct _DexterChannelObject DexterChannelObject;
typedef struct _DexterChannelObjectClass DexterChannelObjectClass;

struct _DexterChannelObject
{
  GObject        parent;
  DexterChannel *channel;      /* channel on which this ch. obj. operates */
  GAsyncQueue   *mbox;         /* waitable object for server messages */
  GObject       *sc;           /* servercomm, used only on server-side objects */
  gint           id;           /* unique ch. obj. id assigned during registration */
  glong          wait_timeout; /* microseconds to wait before timing out during wait */
  gint           wait_types;   /* the types of messages we wait on. others are aynch. */
};

struct _DexterChannelObjectClass
{
  GObjectClass parent_class;

  /* Override to implement functionality for received server message.
   * Base implementation frees message, so chain up when done. */
  void (*process_message) (DexterChannelObject   *channel_object,
                           DexterChannelMessage  *message,
                           GError               **error);

  /* Override to implement functionality for channel events. 
   * Base implementation does nothing. */
  void (*channel_event)   (DexterChannelObject   *channel_object,
                           gint                   event);

  void (*reserved1)       (void);
  void (*reserved2)       (void);
  void (*reserved3)       (void);
  void (*reserved4)       (void);
  void (*reserved5)       (void);
  void (*reserved6)       (void);
  void (*reserved7)       (void);
  void (*reserved8)       (void);
};

struct _DexterChannelMessage
{
  gint     msg_type;  /* enum DexterChannelMessageType */ 
  gint     protocol_major;
  gint     protocol_minor;
  gint     peer_id;
  GString *payload;
  union 
  {     /* one of: command, reply or data */
    struct 
    {   /* command message */
      gint   command;
      gint   parameter;
    } cmd;
    struct 
    {   /* reply message */
      gint   command;
      gint   status;
    } rpy;
    struct 
    {   /* data message */
      gint   flags;
      gint   status;
    } dat;
  };
};


GType     dexter_channel_object_get_type          (void) G_GNUC_INTERNAL G_GNUC_CONST;

void      dexter_channel_object_process_message   (DexterChannelObject   *channel_object,
                                                   DexterChannelMessage  *message,
                                                   GError               **error) G_GNUC_INTERNAL;

void      dexter_channel_object_channel_event     (DexterChannelObject   *channel_object,
                                                   gint                   event) G_GNUC_INTERNAL;

void      dexter_channel_object_send_command      (DexterChannelObject   *channel_object,
                                                   gint                   command,
                                                   gint                   parameter,
                                                   GString               *payload,
                                                   GError               **error) G_GNUC_INTERNAL;

DexterChannelMessage 
         *dexter_channel_object_wait_message      (DexterChannelObject *channel_object,
                                                   glong                timeout_usecs) G_GNUC_INTERNAL;

void      dexter_channel_message_free             (DexterChannelMessage  *message) G_GNUC_INTERNAL;

          /* convenience function for the common send->wait->process paradigm */
void      dexter_channel_object_send_wait_process (DexterChannelObject  *channel_object,
                                                   gint                  command,
                                                   gint                  parameter,
                                                   GString              *payload,
                                                   GError              **error) G_GNUC_INTERNAL;

G_END_DECLS

#endif /* __DEXTER_CHANNEL_OBJECT_H__ */
