/* $Id: dexter-config.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTER_CONFIG_H__
#define __DEXTER_CONFIG_H__

#include <glib-object.h>
#include <dexterplugin-iconfig.h>

G_BEGIN_DECLS

#define DEXTER_TYPE_CONFIG              (dexter_config_get_type ())
#define DEXTER_CONFIG(obj)              (G_TYPE_CHECK_INSTANCE_CAST((obj), \
                                            DEXTER_TYPE_CONFIG, DexterConfig))
#define DEXTER_CONFIG_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), \
                                            DEXTER_TYPE_CONFIG, DexterConfigClass))
#define DEXTER_IS_CONFIG(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
                                            DEXTER_TYPE_CONFIG))
#define DEXTER_IS_CONFIG_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), \
                                            DEXTER_TYPE_CONFIG))
#define DEXTER_CONFIG_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), \
                                            DEXTER_TYPE_CONFIG, DexterConfigClass))

typedef struct _DexterConfig DexterConfig;
typedef struct _DexterConfigClass DexterConfigClass;

typedef enum
{
  DEXTER_CONFIG_SETTING_TYPE_BOOLEAN,
  DEXTER_CONFIG_SETTING_TYPE_INT,
  DEXTER_CONFIG_SETTING_TYPE_STRING,
} DexterConfigSettingType;

typedef union
{
  gboolean default_boolean;
  gint default_int;
  gchar *default_string;
  gpointer default_pointer; /* for string defaults of NULL */
} DexterConfigSettingDefault;

typedef struct _dexter_config_setting
{
  const gchar *config_group;
  const gchar *config_key;
  const DexterConfigSettingType setting_type;
  const DexterConfigSettingDefault setting_default;
} DexterConfigSetting;


GType     dexter_config_get_type          (void) G_GNUC_INTERNAL G_GNUC_CONST;

gboolean  dexter_config_load              (DexterConfig          *config,
                                           const gchar           *filename,
                                           GError               **error) G_GNUC_INTERNAL;

gboolean  dexter_config_has_group         (DexterPluginIConfig   *iconfig,
                                           const gchar           *group) G_GNUC_INTERNAL;

gboolean  dexter_config_get_groups        (DexterPluginIConfig   *iconfig,
                                           gchar               ***return_groups,
                                           gsize                 *length) G_GNUC_INTERNAL;

gboolean  dexter_config_get_keys          (DexterPluginIConfig   *iconfig,
                                           const gchar           *group,
                                           gchar               ***return_keys,
                                           gsize                 *length,
                                           GError               **error) G_GNUC_INTERNAL;

gboolean  dexter_config_get_string        (DexterPluginIConfig   *iconfig,
                                           const gchar           *group,
                                           const gchar           *key,
                                           gchar                **return_string,
                                           GError               **error) G_GNUC_INTERNAL;

gboolean  dexter_config_get_string_list   (DexterPluginIConfig  *iconfig,
                                           const gchar          *group,
                                           const gchar          *key,
                                           gchar              ***return_strings,
                                           gsize                *length,
                                           GError              **error) G_GNUC_INTERNAL;

gboolean  dexter_config_get_boolean       (DexterPluginIConfig  *iconfig,
                                           const gchar          *group,
                                           const gchar          *key,
                                           gboolean             *return_boolean,
                                           GError              **error) G_GNUC_INTERNAL;

gboolean  dexter_config_get_boolean_list  (DexterPluginIConfig  *iconfig,
                                           const gchar          *group,
                                           const gchar          *key,
                                           gboolean            **return_booleans,
                                           gsize                *length,
                                           GError              **error) G_GNUC_INTERNAL;

gboolean  dexter_config_get_integer       (DexterPluginIConfig  *iconfig,
                                           const gchar          *group,
                                           const gchar          *key,
                                           gint                 *return_int,
                                           GError              **error) G_GNUC_INTERNAL;

gboolean  dexter_config_get_integer_list  (DexterPluginIConfig  *iconfig,
                                           const gchar          *group,
                                           const gchar          *key,
                                           gint                **return_integers,
                                           gsize                *length,
                                           GError              **error) G_GNUC_INTERNAL;

gboolean  dexter_config_key_error         (DexterPluginIConfig  *iconfig,
                                           GError               *error) G_GNUC_INTERNAL;

gboolean  dexter_config_get_setting       (DexterPluginIConfig        *iconfig,
                                           const gchar                *group,
                                           const gchar                *key,
                                           DexterConfigSettingType     setting_type,
                                           DexterConfigSettingDefault  setting_default,
                                           DexterConfigSettingDefault *return_setting) G_GNUC_INTERNAL;

G_END_DECLS

#endif /* __DEXTER_CONFIG_H__ */
