/* $Id: dexter-hash.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTER_HASH_H__
#define __DEXTER_HASH_H__

#include <glib.h>

G_BEGIN_DECLS

/* DexterHash is an opaque structure that implements a thread-safe hash table.*/ 
typedef struct _dexter_hash_table DexterHashTable;

DexterHashTable *dexter_hash_table_new             (GHashFunc        hash_func,
                                                    GEqualFunc       key_equal_func) G_GNUC_INTERNAL G_GNUC_MALLOC; 
               
DexterHashTable *dexter_hash_table_new_full        (GHashFunc        hash_func,
                                                    GEqualFunc       key_equal_func,
                                                    GDestroyNotify   key_destroy_func,
                                                    GDestroyNotify   value_destroy_func) G_GNUC_INTERNAL G_GNUC_MALLOC;

void             dexter_hash_table_insert          (DexterHashTable *hash_table,
                                                    gpointer         key,
                                                    gpointer         value) G_GNUC_INTERNAL;

void             dexter_hash_table_replace         (DexterHashTable *hash_table,
                                                    gpointer         key,
                                                    gpointer         value) G_GNUC_INTERNAL;

gint             dexter_hash_table_size            (DexterHashTable *hash_table) G_GNUC_INTERNAL;

gpointer         dexter_hash_table_lookup          (DexterHashTable *hash_table,
                                                    gconstpointer    key) G_GNUC_INTERNAL;

gboolean         dexter_hash_table_lookup_extended (DexterHashTable *hash_table,
                                                    gconstpointer    lookup_key,
                                                    gpointer        *orig_key,
                                                    gpointer        *value) G_GNUC_INTERNAL;

void             dexter_hash_table_foreach         (DexterHashTable *hash_table,
                                                    GHFunc           func,
                                                    gpointer         user_data) G_GNUC_INTERNAL;

gpointer         dexter_hash_table_find            (DexterHashTable *hash_table,
                                                    GHRFunc          predicate,
                                                    gpointer         user_data) G_GNUC_INTERNAL;

gboolean         dexter_hash_table_remove          (DexterHashTable *hash_table,
                                                    gconstpointer    key) G_GNUC_INTERNAL;

gboolean         dexter_hash_table_steal           (DexterHashTable *hash_table,
                                                    gconstpointer    key) G_GNUC_INTERNAL;

gint             dexter_hash_table_foreach_remove  (DexterHashTable *hash_table,
                                                    GHRFunc          func,
                                                    gpointer         user_data) G_GNUC_INTERNAL;

gint             dexter_hash_table_foreach_steal   (DexterHashTable *hash_table,
                                                    GHRFunc          func,
                                                    gpointer         user_data) G_GNUC_INTERNAL;

void             dexter_hash_table_destroy         (DexterHashTable *hash_table) G_GNUC_INTERNAL;

G_END_DECLS

#endif /* __DEXTER_HASH_H__ */
