/* $Id: dexter-messages.c 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <dexter-messages.h>
#include <dexter-alias.h>

static const gchar *dexter_protocol_error[] =
{
  "Success",
  "Bad message format",
  "Incompatible dexter protocols",
  "Protocol greeting not exchanged",
  "Protocol greeting already exchanged",
  "No payload data",
  "Payload data invalid",
  "Service unavailable",
  "Command unknown",
  "Command failed",
  "Invalid command parameter",
  "Sampler peer not found",
  "Sampler data error",
  "TLS not started",
  "TLS already started",
  "TLS not available"
};

const gchar *
dexter_protocol_strerror (gint code)
{
  gint idx;

  switch (code)
  {
    case STATUS_OK:
      idx=0;
      break;
    case ERROR_MESSAGE_BAD_FORMAT:
    case ERROR_INCOMPATIBLE_PROTOCOL:
    case ERROR_GREETING_NOT_EXCHANGED:
    case ERROR_GREETING_ALREADY_EXCHANGED:
    case ERROR_NO_PAYLOAD_DATA:
    case ERROR_PAYLOAD_DATA_INVALID:
    case ERROR_SERVICE_UNAVAILABLE:
    case ERROR_COMMAND_UNKNOWN:
    case ERROR_COMMAND_FAILED:
    case ERROR_COMMAND_INVALID_PARAMETER:
    case ERROR_SAMPLER_PEER_NOT_FOUND:
    case ERROR_SAMPLER_DATA:
    case ERROR_TLS_NOT_STARTED:
    case ERROR_TLS_ALREADY_STARTED:
    case ERROR_TLS_NOT_AVAILABLE:
      idx=(code-9000)/10;
      break;
    default:
      /* programming error */
      g_error (G_STRLOC " unknown protocol error code seen (%d)", code);
      return NULL;
  }

  return dexter_protocol_error [idx];
}

#define __DEXTER_MESSAGESC__
#include <dexter-aliasdef.c>
