/* $Id: dexter-msgframer.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTER_MSGFRAMER_H__
#define __DEXTER_MSGFRAMER_H__

#include <glib-object.h>

G_BEGIN_DECLS

typedef enum
{
  DEXTER_MSGFRAMER_ERROR_BUFFER_OVERRUN   = -1,   /* message content overruns buffers */
  DEXTER_MSGFRAMER_ERROR_CONVERSION       = -2,   /* error during a conversion */
  DEXTER_MSGFRAMER_ERROR_MESSAGE          = -4,   /* message doesn't conform to DEXTER message API */
  DEXTER_MSGFRAMER_ERROR_RANGE            = -8,   /* message value is outside allowed range */
  DEXTER_MSGFRAMER_ERROR_FAILED           = -16   /* other failure */
} DexterMsgFramerError;

typedef struct _dexter_msgframe
{
  gchar    message[64];    /* null-terminated */
  GString *payload;

} DexterMsgFrame;

#define DEXTER_TYPE_MSGFRAMER             (dexter_msgframer_get_type ())
#define DEXTER_MSGFRAMER(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj), \
                                              DEXTER_TYPE_MSGFRAMER, DexterMsgFramer))
#define DEXTER_MSGFRAMER_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), \
                                              DEXTER_TYPE_MSGFRAMER, DexterMsgFramerClass))
#define DEXTER_IS_MSGFRAMER(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
                                              DEXTER_TYPE_MSGFRAMER))
#define DEXTER_IS_MSGFRAMER_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), \
                                              DEXTER_TYPE_MSGFRAMER))
#define DEXTER_MSGFRAMER_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), \
                                              DEXTER_TYPE_MSGFRAMER, DexterMsgFramerClass))

typedef struct _DexterMsgFramer DexterMsgFramer;
typedef struct _DexterMsgFramerClass DexterMsgFramerClass;

GType           dexter_msgframer_get_type      (void) G_GNUC_INTERNAL G_GNUC_CONST;

                /* takes a buffer of buffersize bytes and converts them into queued message frames */
gint            dexter_msgframer_push_bytes    (DexterMsgFramer *framer,
                                                gchar           *buffer,
                                                guint            buffersize) G_GNUC_INTERNAL;
                /* returns number of unprocessed frames in the queue */
gint            dexter_msgframer_frames_queued (DexterMsgFramer *framer) G_GNUC_INTERNAL;

                /* pops the next frame off the queue for processing.  caller must free the frame */
DexterMsgFrame *dexter_msgframer_pop_frame     (DexterMsgFramer *framer) G_GNUC_INTERNAL;

                /* free a message frame */
void            dexter_msgframer_free_frame    (DexterMsgFramer *framer,
                                                DexterMsgFrame  *frame) G_GNUC_INTERNAL;

G_END_DECLS

#endif /* __DEXTER_MSGFRAMER_H__ */
