/* $Id: dexter-network.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTER_NETWORK_H__
#define __DEXTER_NETWORK_H__

#include <sys/types.h>
#include <sys/socket.h>
#include <glib.h>

G_BEGIN_DECLS

typedef enum
{
    DEXTER_NETWORK_ERROR_EACCES,          /* address is protected */
    DEXTER_NETWORK_ERROR_EADDRINUSE,      /* address in use */
    DEXTER_NETWORK_ERROR_EAGAIN,          /* temp resource failure */
    DEXTER_NETWORK_ERROR_ECONNREFUSED,    /* connection refused */
    DEXTER_NETWORK_ERROR_EINTR,           /* system interrupt before completion */
    DEXTER_NETWORK_ERROR_ENETUNREACH,     /* network unreachable */
    DEXTER_NETWORK_ERROR_ETIMEDOUT,       /* remote server timed out */
    DEXTER_NETWORK_ERROR_TLS_NOT_ENABLED, /* TLS not enabled on this installation */
    DEXTER_NETWORK_ERROR_FAILED           /* other failure */
} DexterNetworkError;

typedef struct _dexter_network_session_t dexter_network_session_t;

#define DEXTER_NETWORK_ERROR            dexter_network_error_quark()
GQuark dexter_network_error_quark (void) G_GNUC_INTERNAL;


int      dexter_network_connect              (const char                *hostname,
                                              const char                *service,
                                              const char                *socket_options,
                                              struct sockaddr_storage   *addr,
                                              socklen_t                 *addrlen,
                                              GError                   **error) G_GNUC_INTERNAL;

int      dexter_network_listen               (const char                *hostname,
                                              const char                *service,
                                              const char                *socket_options,
                                              struct sockaddr_storage   *addr,
                                              socklen_t                 *addrlen,
                                              GError                   **error) G_GNUC_INTERNAL;

void     dexter_network_print_socket_options (int                        sockfd) G_GNUC_INTERNAL;

dexter_network_session_t
        *dexter_network_session_new          (void) G_GNUC_INTERNAL G_GNUC_MALLOC;

gboolean dexter_network_session_open         (dexter_network_session_t  *session,
                                              int                        sockfd,
                                              struct sockaddr_storage   *addr,
                                              socklen_t                  addrlen) G_GNUC_INTERNAL;
                        
gboolean dexter_network_session_client_starttls 
                                             (dexter_network_session_t  *session,
                                              const gchar               *cert_file,
                                              const gchar               *key_file,
                                              const gchar               *trust_file,
                                              const gchar               *crl_file,
                                              gboolean                   verify_hostname,
                                              GError                   **error) G_GNUC_INTERNAL;

gboolean dexter_network_session_server_starttls
                                             (dexter_network_session_t  *session,
                                              const gchar               *cert_file,
                                              const gchar               *key_file,
                                              const gchar               *trust_file,
                                              const gchar               *crl_file,
                                              guint                      client_cert_flag,
                                              gboolean                   verify_hostname,
                                              GError                   **error) G_GNUC_INTERNAL;

gboolean dexter_network_session_tls_running  (dexter_network_session_t  *session) G_GNUC_INTERNAL;

const struct sockaddr *
         dexter_network_session_peer_addr    (dexter_network_session_t  *session) G_GNUC_INTERNAL;

socklen_t
         dexter_network_session_peer_addrlen (dexter_network_session_t  *session) G_GNUC_INTERNAL;

gboolean dexter_network_session_pending      (dexter_network_session_t  *session) G_GNUC_INTERNAL;

ssize_t  dexter_network_session_recv         (dexter_network_session_t  *session,
                                              void                      *buffer,
                                              size_t                     bufsize,
                                              int                        flags,
                                              GError                   **error) G_GNUC_INTERNAL;

ssize_t  dexter_network_session_sendall      (dexter_network_session_t  *session,
                                              const void                *buffer,
                                              size_t                     bufsize,
                                              int                        flags,
                                              GError                   **error) G_GNUC_INTERNAL;

void     dexter_network_session_close        (dexter_network_session_t  *session) G_GNUC_INTERNAL;

void     dexter_network_session_free         (dexter_network_session_t  *session) G_GNUC_INTERNAL;

void     dexter_network_session_debug_opts   (dexter_network_session_t  *session) G_GNUC_INTERNAL;
  
G_END_DECLS

#endif /* __DEXTER_NETWORK_H__ */
