/* $Id: dexter-private.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTER_PRIVATE_H__
#define __DEXTER_PRIVATE_H__

#include <glib.h>
#include <dexter-config.h>
#include <dexter-event-loop.h>
#include <dexter-factory.h>
#include <dexter-threadpool.h>
#include <dexter-version.h>

G_BEGIN_DECLS

typedef struct _DexterPrivate DexterPrivate;

struct _DexterPrivate
{
  DexterConfig      *config;
  DexterEventLoop   *eventloop;
  DexterFactory     *factory;
  DexterThreadPool  *threadpool;
  DexterVersion     *version;
};

gboolean             dexter_private_init      (const gchar  *config_filename,
                                               GError      **error) G_GNUC_INTERNAL;

void                 dexter_private_exit      (void) G_GNUC_INTERNAL;

const DexterPrivate *dexter_private_get_priv  (void) G_GNUC_INTERNAL G_GNUC_CONST;

/* Helper macros */
#define __EVENTLOOP     dexter_private_get_priv ()->eventloop
#define __FACTORY       dexter_private_get_priv ()->factory
#define __ICONFIG       DEXTERPLUGIN_ICONFIG (dexter_private_get_priv ()->config)
#define __ITHREADPOOL   DEXTERPLUGIN_ITHREADPOOL (dexter_private_get_priv ()->threadpool)
#define __IVERSION      DEXTERPLUGIN_IVERSION (dexter_private_get_priv ()->version)

G_END_DECLS

#endif /* __DEXTER_PRIVATE_H__ */
