/* $Id: dexter-sampler-internal.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTER_SAMPLER_INTERNAL_H__
#define __DEXTER_SAMPLER_INTERNAL_H__

#include <glib-object.h>
#include <dexter-sampler.h>
#include <dexter-channel-object.h>
#include <dexterplugin-ithreadpool.h>

G_BEGIN_DECLS

#define DEXTER_TYPE_SAMPLER             (dexter_sampler_get_type ())
#define DEXTER_SAMPLER(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj), \
                                              DEXTER_TYPE_SAMPLER, DexterSampler))
#define DEXTER_SAMPLER_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), \
                                              DEXTER_TYPE_SAMPLER, DexterSamplerClass))
#define DEXTER_IS_SAMPLER(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DEXTER_TYPE_SAMPLER))
#define DEXTER_IS_SAMPLER_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), DEXTER_TYPE_SAMPLER))
#define DEXTER_SAMPLER_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), \
                                              DEXTER_TYPE_SAMPLER, DexterSamplerClass))

typedef struct _DexterSamplerClass DexterSamplerClass;
typedef struct _DexterSamplerPrivate DexterSamplerPrivate;

struct _DexterSampler
{
  DexterChannelObject  parent;
  gboolean             dispose_has_run;
  volatile gint        initialized;
  GObject             *plugin_sampler;  /* actual sampler */
  GMutex              *mutex;           /* thread-safety */
};

struct _DexterSamplerClass 
{
  DexterChannelObjectClass parent_class;
};

GType           dexter_sampler_get_type           (void) G_GNUC_INTERNAL G_GNUC_CONST; 

gboolean        dexter_sampler_is_initialized     (DexterSampler              *sampler) G_GNUC_INTERNAL;

DexterSampler  *dexter_sampler_new_actual         (const DexterPluginService  *service,
                                                   const DexterSamplerDataCallback *callbacks,
                                                   DexterChannel              *channel,
                                                   DexterPluginIThreadPool    *ithreadpool) G_GNUC_INTERNAL G_GNUC_MALLOC;

gboolean        dexter_sampler_data_serialize     (DexterSampler              *sampler,
                                                   GString                    *dest_buffer,
                                                   gconstpointer               source_data,
                                                   GError                    **error) G_GNUC_INTERNAL;

gpointer        dexter_sampler_data_deserialize   (DexterSampler              *sampler,
                                                   const GString              *source_buffer,
                                                   GError                    **error) G_GNUC_INTERNAL G_GNUC_MALLOC;

gboolean        dexter_sampler_input_serialize    (DexterSampler              *sampler,
                                                   GString                    *dest_buffer,
                                                   gconstpointer               source_input,
                                                   GError                    **error) G_GNUC_INTERNAL;

gpointer        dexter_sampler_input_deserialize  (DexterSampler              *sampler,
                                                   const GString              *source_buffer,
                                                   GError                    **error) G_GNUC_INTERNAL G_GNUC_MALLOC;

gboolean        dexter_sampler_find_local_service (DexterPluginService       **local_service,
                                                   const DexterPluginService  *remote_service) G_GNUC_INTERNAL;

G_END_DECLS

#endif /* __DEXTER_SAMPLER_INTERNAL_H__ */
