/* $Id: dexter-sampler.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTER_SAMPLER_H__
#define __DEXTER_SAMPLER_H__

#include <glib.h>
#include <dexterplugin-service.h>
#include <dexter-channel.h>

G_BEGIN_DECLS

/**
 * DEXTER_SAMPLER_ERROR:
 *
 * The error domain of the libdexter sampler subsystem.
 **/
#define DEXTER_SAMPLER_ERROR dexter_sampler_error_quark()
GQuark dexter_sampler_error_quark (void);

/**
 * DexterSamplerError:
 * @DEXTER_SAMPLER_ERROR_FAILED: generic failure.
 *
 * Error codes for #DexterSampler operations.
 **/
typedef enum
{
    DEXTER_SAMPLER_ERROR_FAILED
} DexterSamplerError;


/**
 * DexterSampler:
 *
 * Opaque type with no public members.
 **/
typedef struct _DexterSampler DexterSampler;


/**
 * DexterSamplerDataCallback:
 * @sampler: #gpointer to sampler that originated the callback.
 * @sampler_data: #gpointer to plugin-defined sampler data.
 *
 * Each sampler callback receives its own copy of the plugin-defined, sampler_data structure.
 *
 * The library will free the sampler_data for you after the callback returns.
 **/
typedef void (*DexterSamplerDataCallback) (gpointer sampler,
                                           gpointer sampler_data);


DexterSampler *dexter_sampler_new        (const DexterPluginService  *service,
                                          const DexterSamplerDataCallback *callbacks,
                                          DexterChannel              *channel,
                                          GError                    **error) G_GNUC_MALLOC;

void           dexter_sampler_free       (DexterSampler              *sampler,
                                          GError                    **error);

gboolean       dexter_sampler_initialize (DexterSampler              *sampler,
                                          const gint                 *param,
                                          gconstpointer               input,
                                          GError                    **error);

gboolean       dexter_sampler_sample     (DexterSampler              *sampler,
                                          GError                    **error);

gpointer       dexter_sampler_data_new   (DexterSampler              *sampler,
                                          gboolean                    populate) G_GNUC_MALLOC;

gpointer       dexter_sampler_data_dup   (DexterSampler              *sampler,
                                          gconstpointer               source_data) G_GNUC_MALLOC;

void           dexter_sampler_data_xfer  (DexterSampler              *sampler,
                                          gpointer                    old_data,
                                          gconstpointer               new_data);

void           dexter_sampler_data_copy  (DexterSampler              *sampler,
                                          gpointer                    dest_data,
                                          gconstpointer               source_data);

void           dexter_sampler_data_free  (DexterSampler              *sampler,
                                          gpointer                    data);

gpointer       dexter_sampler_input_new  (DexterSampler              *sampler) G_GNUC_MALLOC;

void           dexter_sampler_input_free (DexterSampler              *sampler,
                                          gpointer                    input);

G_END_DECLS

#endif /* __DEXTER_SAMPLER_H__ */
