/* $Id: dexter-servercomm.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTER_SERVERCOMM_H__
#define __DEXTER_SERVERCOMM_H__

#include <glib-object.h>
#include <dexter-msgframer.h>
#include <dexter-server-internal.h>
#include <dexterplugin-service.h>

G_BEGIN_DECLS

#define DEXTER_TYPE_SERVERCOMM              (dexter_servercomm_get_type ())
#define DEXTER_SERVERCOMM(obj)              (G_TYPE_CHECK_INSTANCE_CAST((obj), \
                                                DEXTER_TYPE_SERVERCOMM, DexterServerComm))
#define DEXTER_SERVERCOMM_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), \
                                                DEXTER_TYPE_SERVERCOMM, DexterServerCommClass))
#define DEXTER_IS_SERVERCOMM(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
                                                DEXTER_TYPE_SERVERCOMM))
#define DEXTER_IS_SERVERCOMM_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), \
                                                DEXTER_TYPE_SERVERCOMM))
#define DEXTER_SERVERCOMM_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), \
                                                DEXTER_TYPE_SERVERCOMM, DexterServerCommClass))

typedef struct _DexterServerComm DexterServerComm;
typedef struct _DexterServerCommClass DexterServerCommClass;
typedef struct _DexterServerCommPrivate DexterServerCommPrivate;


struct _DexterServerComm
{
  GObject                    parent;
  DexterServer              *server;                 /* server that owns us */
  DexterServerSettings      *settings;               /* copy of server settings */
  dexter_network_session_t  *session;                /* network session */
  gboolean                   greeting_exchanged;     /* true if protocol greeting exchanged */
  DexterPluginServiceGroup  *sampler_service_group;  /* sampler services we offer */
  GHashTable                *sampler_hash;           /* samplers created on behalf of client */
  GAsyncQueue               *dat_queue;              /* delivery queue of DAT messages for client */
  volatile gint              cur_backlog;            /* current data delivery backlog */
#ifdef HAVE_TLS
  DexterServerTLSSettings   *tls_settings;           /* TLS settings  */
#endif
};

struct _DexterServerCommClass
{
  GObjectClass parent_class;
};


GType   dexter_servercomm_get_type        (void) G_GNUC_INTERNAL G_GNUC_CONST;

void    dexter_servercomm_client_frame    (DexterServerComm      *servercomm,
                                           const DexterMsgFrame  *frame,
                                           gboolean              *done,
                                           GError               **error) G_GNUC_INTERNAL;

void    dexter_servercomm_send_data_frame (DexterServerComm     *servercomm,
                                           GError              **error) G_GNUC_INTERNAL;

G_END_DECLS

#endif /* __DEXTER_SERVERCOMM_H__ */
