/* $Id: dexter-service-broker.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTER_SERVICE_BROKER_H__
#define __DEXTER_SERVICE_BROKER_H__

#include <glib.h>
#include <dexter-channel.h>
#include <dexterplugin-service.h>

G_BEGIN_DECLS

/**
 * DEXTER_SERVICE_BROKER_ERROR:
 *
 * The error domain of the libdexter service broker subsystem.
 **/
#define DEXTER_SERVICE_BROKER_ERROR dexter_service_broker_error_quark()
GQuark dexter_service_broker_error_quark (void);

/**
 * DexterServiceBrokerError:
 * @DEXTER_SERVICE_BROKER_ERROR_FAILED: generic failure.
 *
 * Error codes for #DexterServiceBroker operations.
 **/
typedef enum
{
  DEXTER_SERVICE_BROKER_ERROR_FAILED
} DexterServiceBrokerError;


/**
 * DexterServiceBroker:
 *
 * Opaque type with no public members.
 **/
typedef struct _DexterServiceBroker DexterServiceBroker;


/**
 * DexterServiceType:
 * @DEXTER_SERVICE_SAMPLER: flag for requesting sampler services.
 *
 * Or-able flags used in service_type parameter of the dexter_service_broker_get_services() api.
 * <para>(At present only one service type is implemented).</para>
 **/
typedef enum 
{
  DEXTER_SERVICE_SAMPLER = 1<<0
} DexterServiceType;


DexterServiceBroker      *dexter_service_broker_new          (DexterChannel       *channel) G_GNUC_MALLOC;

DexterPluginServiceGroup *dexter_service_broker_get_services (DexterServiceBroker *service_broker,
                                                              gint                 service_type,
                                                              GError             **error) G_GNUC_MALLOC;

void        dexter_service_broker_free                       (DexterServiceBroker *service_broker);

G_END_DECLS

#endif /* __DEXTER_SERVICE_BROKER_H__ */
