/* $Id: dexter-threadpool.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTER_THREADPOOL_H__
#define __DEXTER_THREADPOOL_H__

#include <glib.h>
#include <dexterplugin-ithreadpool.h>

G_BEGIN_DECLS

/* defaults for g_thread_pool_new () */
#define DEFAULT_THREADPOOL_INIT_EXCLUSIVE     TRUE
#define DEFAULT_THREADPOOL_INIT_MAX_THREADS   10      /* "ThreadPoolNumThreads" */
/* defaults for g_thread_pool_exit () */
#define DEFAULT_THREADPOOL_EXIT_IMMEDIATE     TRUE
#define DEFAULT_THREADPOOL_EXIT_WAIT          TRUE    /* "ThreadPoolExitWait" */


#define DEXTER_TYPE_THREADPOOL              (dexter_threadpool_get_type ())
#define DEXTER_THREADPOOL(obj)              (G_TYPE_CHECK_INSTANCE_CAST((obj), \
                                                DEXTER_TYPE_THREADPOOL, DexterThreadPool))
#define DEXTER_THREADPOOL_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), \
                                                DEXTER_TYPE_THREADPOOL, DexterThreadPoolClass))
#define DEXTER_IS_THREADPOOL(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
                                                DEXTER_TYPE_THREADPOOL))
#define DEXTER_IS_THREADPOOL_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), \
                                                DEXTER_TYPE_THREADPOOL))
#define DEXTER_THREADPOOL_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), \
                                                DEXTER_TYPE_THREADPOOL, DexterThreadPoolClass))

typedef struct _DexterThreadPool DexterThreadPool;
typedef struct _DexterThreadPoolClass DexterThreadPoolClass;
typedef struct _DexterThreadPoolPrivate DexterThreadPoolPrivate;

struct _DexterThreadPool 
{
  GObject       parent;

  GThreadPool  *threadpool;
  volatile gint initialized;

  gboolean      exclusive;
  gint          max_threads;
  gboolean      exit_immediate;
  gboolean      exit_wait;
};

struct _DexterThreadPoolClass 
{
  GObjectClass parent_class;
};


GType     dexter_threadpool_get_type        (void) G_GNUC_INTERNAL G_GNUC_CONST;

gboolean  dexter_threadpool_tp_init         (DexterThreadPool         *threadpool,
                                             GError                  **error) G_GNUC_INTERNAL;

gboolean  dexter_threadpool_is_initialized  (DexterThreadPool         *threadpool) G_GNUC_INTERNAL;

gboolean  dexter_threadpool_push_task       (DexterPluginIThreadPool  *ithreadpool,
                                             GThreadFunc               func,
                                             gpointer                  data,
                                             GError                  **error) G_GNUC_INTERNAL;

void      dexter_threadpool_tp_exit         (DexterThreadPool         *threadpool) G_GNUC_INTERNAL;

G_END_DECLS

#endif /* __DEXTER_THREADPOOL_H__ */
