/* $Id: dexter-timedsampler-internal.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTER_TIMEDSAMPLER_INTERNAL_H__
#define __DEXTER_TIMEDSAMPLER_INTERNAL_H__

#include <dexter-sampler-internal.h>
#include <dexter-timedsampler.h>
#include <dexterplugin-ithreadpool.h>

G_BEGIN_DECLS

#define DEXTER_TYPE_TIMEDSAMPLER            (dexter_timedsampler_get_type ())
#define DEXTER_TIMEDSAMPLER(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), \
                                                  DEXTER_TYPE_TIMEDSAMPLER, DexterTimedSampler))
#define DEXTER_TIMEDSAMPLER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), \
                                                  DEXTER_TYPE_TIMEDSAMPLER, DexterTimedSamplerClass))
#define DEXTER_IS_TIMEDSAMPLER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
                                                  DEXTER_TYPE_TIMEDSAMPLER))
#define DEXTER_IS_TIMEDSAMPLER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), \
                                                  DEXTER_TYPE_TIMEDSAMPLER))
#define DEXTER_TIMEDSAMPLER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), \
                                                  DEXTER_TYPE_TIMEDSAMPLER, DexterTimedSamplerClass))

typedef struct _DexterTimedSamplerClass DexterTimedSamplerClass;
typedef struct _DexterTimedSamplerPrivate DexterTimedSamplerPrivate;

struct _DexterTimedSampler
{
  DexterChannelObject  parent;
  gboolean             dispose_has_run;
  volatile gint        initialized;
  volatile gint        state;
  GObject             *plugin_sampler;  /* actual sampler */
  guint                msecs_interval;  /* milliseconds */
  GMutex              *mutex;           /* thread-safety */
  GSource             *source;          /* timeout source for event loop */
};

struct _DexterTimedSamplerClass {
  DexterChannelObjectClass parent_class;
};

GType               dexter_timedsampler_get_type          (void) G_GNUC_INTERNAL G_GNUC_CONST;

gboolean            dexter_timedsampler_is_initialized    (DexterTimedSampler         *tsampler) G_GNUC_INTERNAL;

DexterTimedSampler *dexter_timedsampler_new_actual        (const DexterPluginService   *service,
                                                           glong                        usecs_interval,
                                                           const DexterSamplerDataCallback *callbacks,
                                                           DexterChannel               *channel,
                                                           DexterPluginIThreadPool     *ithreadpool) G_GNUC_INTERNAL G_GNUC_MALLOC;

gboolean            dexter_timedsampler_data_serialize    (DexterTimedSampler          *tsampler,
                                                           GString                     *dest_buffer,
                                                           gconstpointer                source_data,
                                                           GError                     **error) G_GNUC_INTERNAL;

gpointer            dexter_timedsampler_data_deserialize  (DexterTimedSampler          *tsampler,
                                                           const GString               *source_buffer,
                                                           GError                     **error) G_GNUC_INTERNAL G_GNUC_MALLOC;

gboolean            dexter_timedsampler_input_serialize   (DexterTimedSampler          *tsampler,
                                                           GString                     *dest_buffer,
                                                           gconstpointer                source_input,
                                                           GError                     **error) G_GNUC_INTERNAL;

gpointer            dexter_timedsampler_input_deserialize (DexterTimedSampler          *tsampler,
                                                           const GString               *source_buffer,
                                                           GError                     **error) G_GNUC_INTERNAL G_GNUC_MALLOC;

G_END_DECLS

#endif /* __DEXTER_TIMEDSAMPLER_INTERNAL_H__ */
