/* $Id: dexter-timedsampler.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTER_TIMEDSAMPLER_H__
#define __DEXTER_TIMEDSAMPLER_H__

#include <dexter-sampler.h>

G_BEGIN_DECLS

/**
 * DEXTER_TIMEDSAMPLER_ERROR:
 *
 * The error domain of the libdexter timed sampler subsystem.
 **/
#define DEXTER_TIMEDSAMPLER_ERROR dexter_timedsampler_error_quark()
GQuark dexter_timedsampler_error_quark (void);

/**
 * DexterTimedSamplerError:
 * @DEXTER_TIMEDSAMPLER_ERROR_STARTED: operation not allowed on started timed sampler.
 * @DEXTER_TIMEDSAMPLER_ERROR_NOT_STARTED: operation requires started timed sampler.
 * @DEXTER_TIMEDSAMPLER_ERROR_FAILED: other failure.  
 *
 * Error codes for #DexterTimedSampler operations.
 **/
typedef enum
{
  DEXTER_TIMEDSAMPLER_ERROR_STARTED,
  DEXTER_TIMEDSAMPLER_ERROR_NOT_STARTED,
  DEXTER_TIMEDSAMPLER_ERROR_FAILED
} DexterTimedSamplerError;


/**
 * DexterTimedSampler:
 *
 * Opaque type with no public members.
 **/
typedef struct _DexterTimedSampler DexterTimedSampler;


/**
 * DexterTimedSamplerState:
 * @DEXTER_TIMEDSAMPLER_STATE_STARTED: timed sampler is started.
 * @DEXTER_TIMEDSAMPLER_STATE_STOPPED: timed sampler is stopped.
 *
 * All possible timed sampler states.
 **/
typedef enum
{
  DEXTER_TIMEDSAMPLER_STATE_STARTED,
  DEXTER_TIMEDSAMPLER_STATE_STOPPED,
} DexterTimedSamplerState;


DexterTimedSampler *dexter_timedsampler_new         (const DexterPluginService  *service,
                                                     guint                       msecs_interval,
                                                     const DexterSamplerDataCallback *callbacks,
                                                     DexterChannel              *channel,
                                                     GError                    **error) G_GNUC_MALLOC;

void                dexter_timedsampler_free        (DexterTimedSampler         *tsampler,
                                                     GError                    **error);

gboolean            dexter_timedsampler_initialize  (DexterTimedSampler         *tsampler,
                                                     const gint                 *param,
                                                     gconstpointer               input,
                                                     GError                    **error);

gboolean            dexter_timedsampler_start       (DexterTimedSampler         *tsampler,
                                                     GError                    **error);

gboolean            dexter_timedsampler_stop        (DexterTimedSampler         *tsampler,
                                                     GError                    **error);

gpointer            dexter_timedsampler_data_new    (DexterTimedSampler         *tsampler,
                                                     gboolean                    populate) G_GNUC_MALLOC;

gpointer            dexter_timedsampler_data_dup    (DexterTimedSampler         *tsampler,
                                                     gconstpointer               source_data) G_GNUC_MALLOC;

void                dexter_timedsampler_data_xfer   (DexterTimedSampler         *tsampler,
                                                     gpointer                    old_data,
                                                     gconstpointer               new_data);

void                dexter_timedsampler_data_copy   (DexterTimedSampler         *tsampler,
                                                     gpointer                    dest_data,
                                                     gconstpointer               source_data);

void                dexter_timedsampler_data_free   (DexterTimedSampler         *tsampler,
                                                     gpointer                    data);

gpointer            dexter_timedsampler_input_new   (DexterTimedSampler         *tsampler) G_GNUC_MALLOC;

void                dexter_timedsampler_input_free  (DexterTimedSampler         *tsampler,
                                                     gpointer                    input);

gint                dexter_timedsampler_get_state   (DexterTimedSampler         *tsampler);

G_END_DECLS

#endif /* __DEXTER_TIMEDSAMPLER_H__ */
