/* $Id: dexter-tls.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 2.1 
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
*/

#ifndef __DEXTER_TLS_H__
#define __DEXTER_TLS_H__

#include <sys/types.h>
#include <glib.h>

G_BEGIN_DECLS

#define DEXTER_TLS_ROLE_CLIENT  TRUE
#define DEXTER_TLS_ROLE_SERVER  FALSE

typedef enum
{
  DEXTER_TLS_CERT_NONE          = 1<<0,  /* for server use only: if not asking for client cert */
  DEXTER_TLS_CERT_REQUEST       = 1<<1,  /* for server use only: if requesting client cert */
  DEXTER_TLS_CERT_REQUIRE       = 1<<2,  /* for server use only: if requiring client cert */
  DEXTER_TLS_VERIFY_HOSTNAME    = 1<<3   /* check if dns name matches certificate */
} DexterTLSFlags;

typedef enum
{
  DEXTER_TLS_ERROR_CERTIFICATE,
  DEXTER_TLS_ERROR_EAGAIN,
  DEXTER_TLS_ERROR_EINTR,
  DEXTER_TLS_ERROR_FILE,
  DEXTER_TLS_ERROR_HANDSHAKE,
  DEXTER_TLS_ERROR_IO,
  DEXTER_TLS_ERROR_FAILED
} DexterTLSError;

#define DEXTER_TLS_ERROR dexter_tls_error_quark()
GQuark dexter_tls_error_quark (void) G_GNUC_INTERNAL;

typedef struct _dexter_tls_t dexter_tls_t;

gboolean      dexter_tls_library_init   (GError      **error) G_GNUC_INTERNAL;

void          dexter_tls_server_seed    (void) G_GNUC_INTERNAL;

dexter_tls_t *dexter_tls_new            (gboolean      role,
                                         const gchar  *cert_file,
                                         const gchar  *key_file,
                                         const gchar  *trust_file,
                                         const gchar  *crl_file,
                                         guint         flags,
                                         GError      **error) G_GNUC_INTERNAL G_GNUC_MALLOC;

gboolean      dexter_tls_handshake      (dexter_tls_t *tls,
                                         int           sockfd,
                                         GError      **error) G_GNUC_INTERNAL;

gboolean      dexter_tls_verify_peer    (dexter_tls_t *tls,
                                         const gchar  *canonical_hostname,
                                         GError      **error) G_GNUC_INTERNAL;

ssize_t       dexter_tls_recv           (dexter_tls_t *tls,
                                         void         *buffer,
                                         size_t        bufsize,
                                         GError      **error) G_GNUC_INTERNAL;

ssize_t       dexter_tls_send           (dexter_tls_t *tls,
                                         const void   *buffer,
                                         size_t        bufsize,
                                         GError      **error) G_GNUC_INTERNAL;

void          dexter_tls_close          (dexter_tls_t *tls) G_GNUC_INTERNAL;

void          dexter_tls_free           (dexter_tls_t *tls) G_GNUC_INTERNAL;

void          dexter_tls_library_exit   (void) G_GNUC_INTERNAL;

G_END_DECLS

#endif /* __DEXTER_TLS_H__ */
