/* $Id: dexter-version.c 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <dexter-version.h>
#include <dexter-alias.h>

static void   dexter_version_class_init     (DexterVersionClass            *klass);
static void   dexter_version_init           (DexterVersion                 *version);
static void   dexter_version_finalize       (GObject                       *object);
static void   dexter_version_iversion_init  (DexterPluginIVersionInterface *iversion);

struct _DexterVersion 
{
  GObject parent;
};

struct _DexterVersionClass 
{
  GObjectClass parent_class;
};


G_DEFINE_TYPE_WITH_CODE (DexterVersion, dexter_version, G_TYPE_OBJECT,
                         G_IMPLEMENT_INTERFACE (DEXTERPLUGIN_TYPE_IVERSION,
                                                dexter_version_iversion_init));


static void
dexter_version_class_init (DexterVersionClass  *klass)
{
  GObjectClass *gobject_class;

#ifdef G_ENABLE_DEBUG
  g_message (G_STRFUNC);
#endif

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize = dexter_version_finalize;
}


static void
dexter_version_init (DexterVersion *version)
{
#ifdef G_ENABLE_DEBUG
  g_message (G_STRFUNC);
#endif
}


static void
dexter_version_finalize (GObject *object)
{
  /*DexterVersion *version = DEXTER_VERSION (object);*/

#ifdef G_ENABLE_DEBUG
  g_message (G_STRFUNC);
#endif

  /* chain up */
  G_OBJECT_CLASS (dexter_version_parent_class)->finalize (object);
}


static void
dexter_version_iversion_init (DexterPluginIVersionInterface *iversion)
{
#ifdef G_ENABLE_DEBUG
  g_message (G_STRFUNC);
#endif

  iversion->get_version_major = dexter_version_get_version_major;
  iversion->get_version_minor = dexter_version_get_version_minor;
  iversion->get_version_micro = dexter_version_get_version_micro;
  iversion->check_version_exact = dexter_version_check_version_exact;
  iversion->check_version_compatible = dexter_version_check_version_compatible;
}


guint
dexter_version_get_version_major (DexterPluginIVersion *iversion)
{
  g_return_val_if_fail (DEXTERPLUGIN_IS_IVERSION (iversion), -1);
  return DEXTER_VERSION_MAJOR;
}


guint
dexter_version_get_version_minor (DexterPluginIVersion *iversion)
{
  g_return_val_if_fail (DEXTERPLUGIN_IS_IVERSION (iversion), -1);
  return DEXTER_VERSION_MINOR;
}


guint
dexter_version_get_version_micro (DexterPluginIVersion *iversion)
{
  g_return_val_if_fail (DEXTERPLUGIN_IS_IVERSION (iversion), -1);
  return DEXTER_VERSION_MICRO;
}


void
dexter_version_check_version_exact (DexterPluginIVersion *iversion,
                                    guint                 major, 
                                    guint                 minor, 
                                    guint                 micro)
{
  g_return_if_fail (DEXTERPLUGIN_IS_IVERSION (iversion));
    
  if ( (major != DEXTER_VERSION_MAJOR) ||
       (minor != DEXTER_VERSION_MINOR) ||
       (micro != DEXTER_VERSION_MICRO) )
  {
    /* throw a fatal error on version mismatch */
    g_error ("libdexter version (%d.%d.%d) does not exactly match plugin requirement: (%d.%d.%d)",
             DEXTER_VERSION_MAJOR, DEXTER_VERSION_MINOR, DEXTER_VERSION_MICRO,
             major, minor, micro);
  } 
}


void
dexter_version_check_version_compatible (DexterPluginIVersion *iversion,
                                         guint                 major,
                                         guint                 minor,
                                         guint                 micro)
{
  g_return_if_fail (DEXTERPLUGIN_IS_IVERSION (iversion));

  /* micro version level is ignored */
  if ( (major > DEXTER_VERSION_MAJOR)  ||
       ( (major == DEXTER_VERSION_MAJOR) && (minor > DEXTER_VERSION_MINOR) )) 
  {
    /* throw a fatal error on incompatible error */
    g_error ("libdexter version (%d.%d.%d) not compatible with plugin requirement: (%d.%d.%d)",
             DEXTER_VERSION_MAJOR, DEXTER_VERSION_MINOR, DEXTER_VERSION_MICRO,
             major, minor, micro);
  }
}

#define __DEXTER_VERSION_C__
#include <dexter-aliasdef.c>
