/* $Id: dexter.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; either version 2.1 
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
*/

#ifndef __DEXTER_H__
#define __DEXTER_H__

#include <glib.h>
#include <dexter-channel.h>
#include <dexter-sampler.h>
#include <dexter-server.h>
#include <dexter-service-broker.h>
#include <dexter-timedsampler.h>

G_BEGIN_DECLS

/**
 * DEXTER_TLS_ENABLED:
 *
 * This macro is defined if TLS is enabled on this installation of Libdexter.
 *
 * Since: 0.2.0
 **/
#ifdef HAVE_TLS
#define DEXTER_TLS_ENABLED
#endif

/**
 * DEXTER_ERROR:
 *
 * The error domain of the libdexter initialization subsystem.
 **/
#define DEXTER_ERROR dexter_error_quark()
GQuark dexter_error_quark (void);

/**
 * DexterError:
 * @DEXTER_ERROR_NO_CONFIG_FILE: no configuration file was indicated or found.
 * @DEXTER_ERROR_MODULES_NOT_SUPPORTED: installation does not support loadable modules.
 * @DEXTER_ERROR_NOT_ABSOLUTE_PATH: operation requires specification of an absolute path.
 * @DEXTER_ERROR_FAILED: other failure.
 *
 * Error codes for dexter operations.
 **/
typedef enum
{
  DEXTER_ERROR_NO_CONFIG_FILE,
  DEXTER_ERROR_MODULES_NOT_SUPPORTED,
  DEXTER_ERROR_NOT_ABSOLUTE_PATH,
  DEXTER_ERROR_FAILED
} DexterError;


/**
 * DexterState:
 * @DEXTER_STATE_UP: the library is up and available for use.
 * @DEXTER_STATE_STARTING: the library is starting, i.e. dexter_init() is in progress.
 * @DEXTER_STATE_STOPPING: the library is shutting down, i.e. dexter_exit() is in progress.
 * @DEXTER_STATE_DOWN: the library is down and not available for use.
 *
 * All possible states for the Libdexter library.
 **/
typedef enum
{
  DEXTER_STATE_UP,
  DEXTER_STATE_STARTING,
  DEXTER_STATE_STOPPING,
  DEXTER_STATE_DOWN
} DexterState;


gboolean  dexter_init (const gchar  *config,
                       GError      **error);

void      dexter_exit (void);

gint      dexter_get_state (void);

G_END_DECLS

#endif /* __DEXTER_H__ */
