/* $Id: dexterplugin-iconfig.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTERPLUGIN_ICONFIG_H__
#define __DEXTERPLUGIN_ICONFIG_H__

#include <glib-object.h>

G_BEGIN_DECLS

#define DEXTERPLUGIN_TYPE_ICONFIG                 (dexterplugin_iconfig_get_type ())
#define DEXTERPLUGIN_ICONFIG(obj)                 (G_TYPE_CHECK_INSTANCE_CAST((obj), \
                                                      DEXTERPLUGIN_TYPE_ICONFIG, \
                                                      DexterPluginIConfig))
#define DEXTERPLUGIN_IS_ICONFIG(obj)              (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
                                                      DEXTERPLUGIN_TYPE_ICONFIG))
#define DEXTERPLUGIN_ICONFIG_GET_INTERFACE(obj)   (G_TYPE_INSTANCE_GET_INTERFACE ((obj), \
                                                      DEXTERPLUGIN_TYPE_ICONFIG, \
                                                      DexterPluginIConfigInterface))

/**
 * DexterPluginIConfig:
 *
 * The configuration file interface.
 **/
typedef struct _DexterPluginIConfig DexterPluginIConfig;
typedef struct _DexterPluginIConfigInterface DexterPluginIConfigInterface;

/**
 * DexterPluginIConfigInterface:
 * @has_group: See dexterplugin_iconfig_has_group().
 * @get_groups: See dexterplugin_iconfig_get_groups().
 * @get_keys: See dexterplugin_iconfig_get_keys().
 * @get_string: See dexterplugin_iconfig_get_string().
 * @get_string_list: See dexterplugin_iconfig_get_string_list().
 * @get_boolean: See dexterplugin_iconfig_get_boolean().
 * @get_boolean_list: See dexterplugin_iconfig_get_boolean_list().
 * @get_integer: See dexterplugin_iconfig_get_integer().
 * @get_integer_list: See dexterplugin_iconfig_get_integer_list().
 * @key_error: See dexterplugin_iconfig_key_error().
 *
 * Definition of the configuration file interface.
 **/
struct _DexterPluginIConfigInterface 
{
  /*< private >*/
  GTypeInterface   parent_interface;

  /*< public >*/

  /* virtual methods */

  gboolean    (*has_group)        (DexterPluginIConfig   *iconfig,
                                   const gchar           *group);

  gboolean    (*get_groups)       (DexterPluginIConfig   *iconfig,
                                   gchar               ***return_groups,
                                   gsize                 *length);

  gboolean    (*get_keys)         (DexterPluginIConfig   *iconfig,
                                   const gchar           *group,
                                   gchar               ***return_keys,
                                   gsize                 *length,
                                   GError               **error);
    
  gboolean    (*get_string)       (DexterPluginIConfig   *iconfig,
                                   const gchar           *group,
                                   const gchar           *key,
                                   gchar                **return_string,
                                   GError               **error);

  gboolean    (*get_string_list)  (DexterPluginIConfig   *iconfig,
                                   const gchar           *group,
                                   const gchar           *key,
                                   gchar               ***return_strings,
                                   gsize                 *length,
                                   GError               **error);

  gboolean    (*get_boolean)      (DexterPluginIConfig   *iconfig,
                                   const gchar           *group,
                                   const gchar           *key,
                                   gboolean              *return_boolean,
                                   GError               **error);

  gboolean    (*get_boolean_list) (DexterPluginIConfig   *iconfig,
                                   const gchar           *group,
                                   const gchar           *key,
                                   gboolean             **return_booleans,
                                   gsize                 *length,
                                   GError               **error);

  gboolean    (*get_integer)      (DexterPluginIConfig   *iconfig,
                                   const gchar           *group,
                                   const gchar           *key,
                                   gint                  *return_integer,
                                   GError               **error);

  gboolean    (*get_integer_list) (DexterPluginIConfig   *iconfig,
                                   const gchar           *group,
                                   const gchar           *key,
                                   gint                 **return_integers,
                                   gsize                 *length,
                                   GError               **error);

  gboolean    (*key_error)        (DexterPluginIConfig   *iconfig,
                                   GError                *error);


  /*< private >*/
  void        (*reserved1)        (void);
  void        (*reserved2)        (void);
  void        (*reserved3)        (void);
  void        (*reserved4)        (void);
  void        (*reserved5)        (void);
  void        (*reserved6)        (void);
  void        (*reserved7)        (void);
  void        (*reserved8)        (void);
};


GType      dexterplugin_iconfig_get_type          (void) G_GNUC_CONST;


gboolean   dexterplugin_iconfig_has_group         (DexterPluginIConfig   *iconfig,
                                                   const gchar           *group);

gboolean   dexterplugin_iconfig_get_groups        (DexterPluginIConfig   *iconfig,
                                                   gchar               ***return_groups,
                                                   gsize                 *length);

gboolean   dexterplugin_iconfig_get_keys          (DexterPluginIConfig   *iconfig,
                                                   const gchar           *group,
                                                   gchar               ***return_keys,
                                                   gsize                 *length,
                                                   GError               **error);

gboolean   dexterplugin_iconfig_get_string        (DexterPluginIConfig   *iconfig,
                                                   const gchar           *group,
                                                   const gchar           *key,
                                                   gchar                **return_string,
                                                   GError               **error);

gboolean   dexterplugin_iconfig_get_string_list   (DexterPluginIConfig   *iconfig,
                                                   const gchar           *group,
                                                   const gchar           *key,
                                                   gchar               ***return_strings,
                                                   gsize                 *length,
                                                   GError               **error);

gboolean   dexterplugin_iconfig_get_boolean       (DexterPluginIConfig   *iconfig,
                                                   const gchar           *group,
                                                   const gchar           *key,
                                                   gboolean              *return_boolean,
                                                   GError               **error);

gboolean   dexterplugin_iconfig_get_boolean_list  (DexterPluginIConfig   *iconfig,
                                                   const gchar           *group,
                                                   const gchar           *key,
                                                   gboolean             **return_booleans,
                                                   gsize                 *length,
                                                   GError               **error);

gboolean   dexterplugin_iconfig_get_integer       (DexterPluginIConfig   *iconfig,
                                                   const gchar           *group,
                                                   const gchar           *key,
                                                   gint                  *return_integer,
                                                   GError               **error);

gboolean   dexterplugin_iconfig_get_integer_list  (DexterPluginIConfig   *iconfig,
                                                   const gchar           *group,
                                                   const gchar           *key,
                                                   gint                 **return_integers,
                                                   gsize                 *length,
                                                   GError               **error);

gboolean   dexterplugin_iconfig_key_error         (DexterPluginIConfig   *iconfig,
                                                   GError                *error);

G_END_DECLS

#endif /* __DEXTERPLUGIN_ICONFIG_H__ */
