/* $Id: dexterplugin-isampler.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTERPLUGIN_ISAMPLER_H__
#define __DEXTERPLUGIN_ISAMPLER_H__

#include <glib-object.h>

G_BEGIN_DECLS

#define DEXTERPLUGIN_TYPE_ISAMPLER               (dexterplugin_isampler_get_type ())
#define DEXTERPLUGIN_ISAMPLER(obj)               (G_TYPE_CHECK_INSTANCE_CAST((obj), \
                                                      DEXTERPLUGIN_TYPE_ISAMPLER, DexterPluginISampler))
#define DEXTERPLUGIN_IS_ISAMPLER(obj)            (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
                                                      DEXTERPLUGIN_TYPE_ISAMPLER))
#define DEXTERPLUGIN_ISAMPLER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), \
                                                      DEXTERPLUGIN_TYPE_ISAMPLER, \
                                                      DexterPluginISamplerInterface))

/**
 * DexterPluginISampler:
 *
 * Dummy object for the DexterPluginISampler interface.
 **/
typedef struct _DexterPluginISampler DexterPluginISampler;
typedef struct _DexterPluginISamplerInterface DexterPluginISamplerInterface;

/**
 * DexterPluginISamplerInterface:
 * @initialize: See dexterplugin_isampler_initialize().
 * @sample: See dexterplugin_isampler_sample().
 * @data_new: See dexterplugin_isampler_data_new().
 * @data_dup: See dexterplugin_isampler_data_dup().
 * @data_xfer: See dexterplugin_isampler_data_xfer().
 * @data_copy: See dexterplugin_isampler_data_copy().
 * @data_free: See dexterplugin_isampler_data_free().
 * @data_serialize: See dexterplugin_isampler_data_serialize().
 * @data_deserialize: See dexterplugin_isampler_data_deserialize().
 * @input_new: See dexterplugin_isampler_input_new().
 * @input_free: See dexterplugin_isampler_input_free().
 * @input_serialize: See dexterplugin_isampler_input_serialize().
 * @input_deserialize: See dexterplugin_isampler_input_deserialize().
 *
 * The definition of the DexterPluginISamplerInterface.
 **/
struct _DexterPluginISamplerInterface
{
  /*< private >*/
  GTypeInterface parent_interface;

  /*< public >*/
  gboolean (*initialize)        (DexterPluginISampler  *isampler,
                                 const gint            *param,
                                 gconstpointer          input,
                                 GError               **error); 

  gboolean (*sample)            (DexterPluginISampler  *isampler,
                                 GError               **error);

  gpointer (*data_new)          (DexterPluginISampler  *isampler,
                                 gboolean               populate);
    
  gpointer (*data_dup)          (DexterPluginISampler  *isampler,
                                 gconstpointer          source_data);

  void     (*data_xfer)         (DexterPluginISampler  *isampler,
                                 gpointer               old_data,
                                 gconstpointer          new_data);
    
  void     (*data_copy)         (DexterPluginISampler  *isampler,
                                 gpointer               dest_data,
                                 gconstpointer          source_data);
    
  void     (*data_free)         (DexterPluginISampler  *isampler,
                                 gpointer               data);

  gboolean (*data_serialize)    (DexterPluginISampler  *isampler,
                                 GString               *dest_buffer,
                                 gconstpointer          source_data,
                                 GError               **error);

  gpointer (*data_deserialize)  (DexterPluginISampler  *isampler,
                                 const GString         *source_buffer,
                                 GError               **error);
                
  gpointer (*input_new)         (DexterPluginISampler  *isampler);

  void     (*input_free)        (DexterPluginISampler  *isampler,
                                 gpointer               input);

  gboolean (*input_serialize)   (DexterPluginISampler  *isampler,
                                 GString               *dest_buffer,
                                 gconstpointer          source_input,
                                 GError               **error);

  gpointer (*input_deserialize) (DexterPluginISampler  *isampler,
                                 const GString         *source_buffer,
                                 GError               **error);

  /*< private >*/
  void     (*reserved1) (void);
  void     (*reserved2) (void);
  void     (*reserved3) (void);
  void     (*reserved4) (void);
  void     (*reserved5) (void);
  void     (*reserved6) (void);
  void     (*reserved7) (void);
  void     (*reserved8) (void);
};

GType      dexterplugin_isampler_get_type          (void) G_GNUC_CONST;

gboolean   dexterplugin_isampler_initialize        (DexterPluginISampler  *isampler,
                                                    const gint            *param,
                                                    gconstpointer          input,
                                                    GError               **error);

gboolean   dexterplugin_isampler_sample            (DexterPluginISampler  *isampler,
                                                    GError               **error);

gpointer   dexterplugin_isampler_data_new          (DexterPluginISampler  *isampler,
                                                    gboolean               populate) G_GNUC_MALLOC;

gpointer   dexterplugin_isampler_data_dup          (DexterPluginISampler  *isampler,
                                                    gconstpointer          source_data) G_GNUC_MALLOC;

void       dexterplugin_isampler_data_xfer         (DexterPluginISampler  *isampler,
                                                    gpointer               old_data,
                                                    gconstpointer          new_data);

void       dexterplugin_isampler_data_copy         (DexterPluginISampler  *isampler,
                                                    gpointer               dest_data,
                                                    gconstpointer          source_data);

void       dexterplugin_isampler_data_free         (DexterPluginISampler  *isampler,
                                                    gpointer               data);

gboolean   dexterplugin_isampler_data_serialize    (DexterPluginISampler  *isampler,
                                                    GString               *dest_buffer,
                                                    gconstpointer          source_data,
                                                    GError               **error);

gpointer   dexterplugin_isampler_data_deserialize  (DexterPluginISampler  *isampler,
                                                    const GString         *source_buffer,
                                                    GError               **error) G_GNUC_MALLOC;

gpointer   dexterplugin_isampler_input_new         (DexterPluginISampler  *isampler) G_GNUC_MALLOC;

void       dexterplugin_isampler_input_free        (DexterPluginISampler  *isampler,
                                                    gpointer               input);

gboolean   dexterplugin_isampler_input_serialize   (DexterPluginISampler  *isampler,
                                                    GString               *dest_buffer,
                                                    gconstpointer          source_input,
                                                    GError               **error);

gpointer   dexterplugin_isampler_input_deserialize (DexterPluginISampler  *isampler,
                                                    const GString         *source_buffer,
                                                    GError               **error) G_GNUC_MALLOC;

G_END_DECLS

#endif /* __DEXTERPLUGIN_ISAMPLER_H__ */
