/* $Id: dexterplugin-ithreadpool.c 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

/*[no gtk-doc]*
 * SECTION:dexterplugin-ithreadpool
 * @title: DexterPluginIThreadPool
 * @short_description: Interface to the Libdexter thread pool
 * @see_also:
 *
 * The sampler base class may use the thread pool via this interface.
 **/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <dexterplugin-ithreadpool.h>
#include <dexterplugin-alias.h>

static void dexterplugin_ithreadpool_base_init (gpointer g_class);

static void
dexterplugin_ithreadpool_base_init (gpointer g_class)
{
  static gboolean initialized = FALSE;

  if (!initialized)
  {
    initialized = TRUE;
  }
}

GType
dexterplugin_ithreadpool_get_type (void)
{
  static GType type_id = G_TYPE_INVALID;

  if (G_UNLIKELY (type_id == G_TYPE_INVALID))
  {
    static const GTypeInfo type_info =
    {
      sizeof (DexterPluginIThreadPoolInterface),
      dexterplugin_ithreadpool_base_init,
      NULL,
      NULL,
      NULL,
      NULL,
      0,
      0,
      NULL,
      NULL,
    };

    type_id = g_type_register_static (G_TYPE_INTERFACE, "DexterPluginIThreadPool", &type_info, 0);
    g_type_interface_add_prerequisite (type_id, G_TYPE_OBJECT);
  }

  return type_id;
}



/*[no gtk-doc]*
 * dexterplugin_ithreadpool_push_task:
 * @ithreadpool: a #DexterPluginIThreadPool.
 * @func: #GThreadFunc to run in the thread.
 * @data: #gpointer data to pass to the thread function.
 * @error: return location for optional #GError.
 *
 * A task consisting of the specified thread function @func called with parameter @data
 * is pushed onto the thread pool.
 *
 * Returns: #gboolean indicating success or failure.
 **/
gboolean
dexterplugin_ithreadpool_push_task (DexterPluginIThreadPool  *ithreadpool,
                                    GThreadFunc               func,
                                    gpointer                  data,
                                    GError                  **error)
{
  g_return_val_if_fail (DEXTERPLUGIN_IS_ITHREADPOOL (ithreadpool), FALSE);
  g_return_val_if_fail (func != NULL, FALSE);
  g_return_val_if_fail (data != NULL, FALSE);
  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
  return DEXTERPLUGIN_ITHREADPOOL_GET_INTERFACE (ithreadpool) -> push_task (ithreadpool, func, data, error);
}

#define __DEXTERPLUGIN_ITHREADPOOL_C__
#include <dexterplugin-aliasdef.c>
