/* $Id: dexterplugin-iversion.h.in 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTERPLUGIN_IVERSION_H__
#define __DEXTERPLUGIN_IVERSION_H__

#include <glib-object.h>

G_BEGIN_DECLS

/**
 * DEXTER_VERSION_MAJOR:
 *
 * The major version of Libdexter.
 **/
#define DEXTER_VERSION_MAJOR    0 

/**
 * DEXTER_VERSION_MINOR:
 *
 * The minor version of Libdexter.
 **/
#define DEXTER_VERSION_MINOR    2

/**
 * DEXTER_VERSION_MICRO:
 *
 * The mirco version of Libdexter.
 **/
#define DEXTER_VERSION_MICRO    1

/**
 * DEXTER_CHECK_VERSION_EXACT:
 *
 * Shortcut macro for calling dexterplugin_iversion_check_version_exact() with required paramaters.
 **/
#define DEXTER_CHECK_VERSION_EXACT \
  dexterplugin_iversion_check_version_exact (dexterplugin_main_imain_get ()->iversion, \
                                             DEXTER_VERSION_MAJOR, \
                                             DEXTER_VERSION_MINOR, \
                                             DEXTER_VERSION_MICRO)

/**
 * DEXTER_CHECK_VERSION_COMPATIBLE:
 *
 * Shortcut macro for calling dexterplugin_iversion_check_version_compatible() with required parameters.
 **/
#define DEXTER_CHECK_VERSION_COMPATIBLE \
  dexterplugin_iversion_check_version_compatible (dexterplugin_main_imain_get ()->iversion, \
                                                  DEXTER_VERSION_MAJOR, \
                                                  DEXTER_VERSION_MINOR, \
                                                  DEXTER_VERSION_MICRO)


#define DEXTERPLUGIN_TYPE_IVERSION                (dexterplugin_iversion_get_type ())
#define DEXTERPLUGIN_IVERSION(obj)                (G_TYPE_CHECK_INSTANCE_CAST((obj), \
                                                      DEXTERPLUGIN_TYPE_IVERSION, \
                                                      DexterPluginIVersion))
#define DEXTERPLUGIN_IS_IVERSION(obj)             (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
                                                      DEXTERPLUGIN_TYPE_IVERSION))
#define DEXTERPLUGIN_IVERSION_GET_INTERFACE(obj)  (G_TYPE_INSTANCE_GET_INTERFACE ((obj), \
                                                      DEXTERPLUGIN_TYPE_IVERSION, \
                                                      DexterPluginIVersionInterface))


/**
 * DexterPluginIVersion:
 *
 * The version information interface.
 **/
typedef struct _DexterPluginIVersion DexterPluginIVersion;
typedef struct _DexterPluginIVersionInterface DexterPluginIVersionInterface;

/**
 * DexterPluginIVersionInterface:
 * @get_version_major: See dexterplugin_iversion_get_version_major().
 * @get_version_minor: See dexterplugin_iversion_get_version_minor().
 * @get_version_micro: See dexterplugin_iversion_get_version_micro().
 * @check_version_exact: See dexterplugin_iversion_check_version_exact().
 * @check_version_compatible: See dexterplugin_iversion_check_version_compatible().
 *
 * Definition of the version information interface.
 **/
struct _DexterPluginIVersionInterface 
{
  /*< private >*/
  GTypeInterface parent_interface;

  /*< public >*/

  /* virtual methods */
  guint    (*get_version_major)       (DexterPluginIVersion *iversion);
  guint    (*get_version_minor)       (DexterPluginIVersion *iversion);
  guint    (*get_version_micro)       (DexterPluginIVersion *iversion);

  void     (*check_version_exact)     (DexterPluginIVersion *iversion,
                                       guint                 major, 
                                       guint                 minor, 
                                       guint                 micro);
    
  void    (*check_version_compatible) (DexterPluginIVersion *iversion,
                                       guint                 major, 
                                       guint                 minor, 
                                       guint                 micro);

  /*< private >*/
  void    (*reserved1)                (void);
  void    (*reserved2)                (void);
  void    (*reserved3)                (void);
  void    (*reserved4)                (void);
  void    (*reserved5)                (void);
  void    (*reserved6)                (void);
  void    (*reserved7)                (void);
  void    (*reserved8)                (void);
};

GType   dexterplugin_iversion_get_type                 (void) G_GNUC_CONST;

guint   dexterplugin_iversion_get_version_major        (DexterPluginIVersion *iversion);

guint   dexterplugin_iversion_get_version_minor        (DexterPluginIVersion *iversion);

guint   dexterplugin_iversion_get_version_micro        (DexterPluginIVersion *iversion);

void    dexterplugin_iversion_check_version_exact      (DexterPluginIVersion *iversion,
                                                        guint                 major, 
                                                        guint                 minor,   
                                                        guint                 micro);

void    dexterplugin_iversion_check_version_compatible (DexterPluginIVersion *iversion,
                                                        guint                 major, 
                                                        guint                 minor, 
                                                        guint                 micro);

G_END_DECLS

#endif
