/* $Id: dexterplugin-sampler.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTERPLUGIN_SAMPLER_H__
#define __DEXTERPLUGIN_SAMPLER_H__

#include <glib-object.h>

G_BEGIN_DECLS

/**
 * DEXTERPLUGIN_SAMPLER_ERROR:
 *
 * The error domain of the libdexterplugin sampler subsystem.
 **/
#define DEXTERPLUGIN_SAMPLER_ERROR dexterplugin_sampler_error_quark()
GQuark dexterplugin_sampler_error_quark (void);

/**
 * DexterPluginSamplerError:
 * @DEXTERPLUGIN_SAMPLER_ERROR_FAILED: general failure.
 *
 * Error codes for #DexterPluginSampler operations.
 **/
typedef enum
{
  DEXTERPLUGIN_SAMPLER_ERROR_FAILED
} DexterPluginSamplerError;


/* abstract base class for all sampler plugins */
#define DEXTERPLUGIN_TYPE_SAMPLER             (dexterplugin_sampler_get_type ())
#define DEXTERPLUGIN_SAMPLER(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj), \
                                                  DEXTERPLUGIN_TYPE_SAMPLER, \
                                                  DexterPluginSampler))
#define DEXTERPLUGIN_SAMPLER_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), \
                                                  DEXTERPLUGIN_TYPE_SAMPLER, \
                                                  DexterPluginSamplerClass))
#define DEXTERPLUGIN_IS_SAMPLER(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
                                                  DEXTERPLUGIN_TYPE_SAMPLER))
#define DEXTERPLUGIN_IS_SAMPLER_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), \
                                                  DEXTERPLUGIN_TYPE_SAMPLER))
#define DEXTERPLUGIN_SAMPLER_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), \
                                                  DEXTERPLUGIN_TYPE_SAMPLER, \
                                                  DexterPluginSamplerClass))


typedef struct _DexterPluginSampler DexterPluginSampler;
typedef struct _DexterPluginSamplerClass DexterPluginSamplerClass;
typedef struct _DexterPluginSamplerPrivate DexterPluginSamplerPrivate;


/**
 * DexterPluginSamplerDataCallback:
 * @sampler: #gpointer to sampler that generated the callback.
 * @sampler_data: #gpointer to sampler data as defined by the plugin.
 *
 * Signature of callbacks executed by plugin samplers to notify users of data changes after sampling.
 **/
typedef void (*DexterPluginSamplerDataCallback) (gpointer sampler,  
                                                 gpointer sampler_data);

/**
 * DexterPluginSamplerClass:
 * @parent_class: parent class.
 *
 * Definition of base class type for sampler services.
 **/
struct _DexterPluginSamplerClass 
{     
  /*< public >*/
  GObjectClass parent_class;
}; 

/**
 * DexterPluginSampler:
 *
 * Definition of base type for sampler services.
 **/
struct _DexterPluginSampler 
{
  /*< public >*/
  GObject parent;

  /*< private >*/
  DexterPluginSamplerPrivate *priv;
};

GType dexterplugin_sampler_get_type  (void) G_GNUC_CONST;

void  dexterplugin_sampler_callbacks (DexterPluginSampler *plugin_sampler);

G_END_DECLS

#endif /* __DEXTERPLUGIN_SAMPLER_H__ */
