/* $Id: dexterplugin-service.h 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifndef __DEXTERPLUGIN_SERVICE_H__
#define __DEXTERPLUGIN_SERVICE_H__

#include <glib-object.h>

G_BEGIN_DECLS

/**
 * DEXTERPLUGIN_SERVICE_ERROR:
 *
 * The error domain of the libdexterplugin service subsystem.
 **/
#define DEXTERPLUGIN_SERVICE_ERROR dexterplugin_service_error_quark()
GQuark dexterplugin_service_error_quark (void);

/**
 * DexterPluginServiceError:
 * @DEXTERPLUGIN_SERVICE_ERROR_FAILED: general failure.
 *
 * Error codes for #DexterPluginService operations.
 **/
typedef enum
{
    DEXTERPLUGIN_SERVICE_ERROR_FAILED
} DexterPluginServiceError;


/**
 * DexterPluginService:
 * @type: service type assigned during service registration.
 * @name: name of the service.
 * @blurb: description of the service.
 * @uuid: univerally unique identifier of the service.
 *
 * A plugin service descriptor.
 **/
typedef struct _dexterplugin_service {
  /*< public >*/
  GType       type;
  gchar      *name;
  gchar      *blurb;
  gchar      *uuid;

} DexterPluginService;


/**
 * DexterPluginServiceGroup:
 * @n_services: number of services in the group.
 * @services: service array of size @n_services.
 *
 * A collection of plugin service descriptors.
 *
 **/
typedef struct _dexterplugin_service_group {
  /*< public >*/
  guint                n_services;
  DexterPluginService *services;

} DexterPluginServiceGroup;


DexterPluginService      *dexterplugin_service_new             (GType                      type,
                                                                const gchar               *name,
                                                                const gchar               *blurb,
                                                                const gchar               *uuid) G_GNUC_MALLOC;

DexterPluginService      *dexterplugin_service_dup             (const DexterPluginService *service) G_GNUC_MALLOC;

void                      dexterplugin_service_copy            (DexterPluginService       *dest,
                                                                const DexterPluginService *source);

void                      dexterplugin_service_free            (DexterPluginService       *service);

DexterPluginServiceGroup *dexterplugin_service_group_new       (guint                      n_services) G_GNUC_MALLOC;

void                      dexterplugin_service_group_free      (DexterPluginServiceGroup  *service_group);

gboolean                  dexterplugin_service_group_find_uuid (DexterPluginService      **service,
                                                                DexterPluginServiceGroup  *service_group,
                                                                const gchar               *uuid);

G_END_DECLS

#endif /* __DEXTERPLUGIN_SERVICE_H__ */
