/* $Id: daemon.c 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * libdexter - a plugin-based, distributed sampling library.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifdef HAVE_CONFIG
#include <config.h>
#endif

#include <fcntl.h>
#include <signal.h>
#include <sys/types.h>
#include <unistd.h>

int daemon (int nochdir, int noclose)
{
  pid_t pid;

  /* fork so setsid() below guaranteed not be run as session leader */
  if ((pid = fork ()) < 0)
    return (-1);
  else if (pid)
    _exit (0);
 
  /* become session leader and fork again so we cannot acquire a controlling terminal  */
  if (setsid() < 0)
    return (-1);

  if (signal (SIGHUP, SIG_IGN) == SIG_ERR)
    return (-1);

  if ((pid = fork ()) < 0)
    return (-1);
  else if (pid)
    _exit (0);

  if ((!nochdir) && (chdir("/") < 0))
    return (-1);

  if (!noclose)
  {
    int dev_null_fd;

    if ((close (0) < 0) || (close (1) < 0) || (close (2) < 0))
      return (-1);

    if ((dev_null_fd = open ("/dev/null", O_RDONLY)) < 0)
      return (-1);
    
    if (dup2 (dev_null_fd, 0) < 0)
      return (-1);

    if ((dev_null_fd = open ("/dev/null", O_RDWR)) < 0)
      return (-1);

    if ((dup2 (dev_null_fd, 1) < 0) || (dup2 (dev_null_fd, 2) < 0))
      return (-1);
  }

  return 0;
}
