/* $Id: dxt-null-export.c 393 2012-03-06 23:36:44Z pkovacs $ */
/*
 * dxt-null - libdexter sampling plugin that does nothing at all.
 * Copyright (C) 2006-2012 Philip Kovacs <pkovacs@users.sourceforge.net>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301  USA
 *
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <dexterplugin.h>
#include <dxt-null.h>

static DexterPluginService registered_services[] = 
{
  { G_TYPE_INVALID,         /* type set during dexter_plugin_init */
    DXT_NULL_SERVICE_NAME,
    DXT_NULL_SERVICE_BLURB,
    DXT_NULL_SERVICE_UUID
  }
};


G_MODULE_EXPORT void
dexter_plugin_init (GTypeModule *module)
{
#ifdef G_ENABLE_DEBUG
  g_message (G_STRFUNC);
#endif

  /* check for version compatibility with the main library */
  DEXTER_CHECK_VERSION_COMPATIBLE;

  dxt_null_register_type (module);

  registered_services[0].type = DXT_TYPE_NULL;
}


G_MODULE_EXPORT void
dexter_plugin_exit (void)
{
#ifdef G_ENABLE_DEBUG
  g_message (G_STRFUNC);
#endif
}


G_MODULE_EXPORT DexterPluginServiceGroup *
dexter_plugin_get_services (void)
{
  DexterPluginServiceGroup *service_group = NULL;
  int iter;

  /* allocate a service group to hold all services this plugin registers */
  service_group = dexterplugin_service_group_new (G_N_ELEMENTS (registered_services));

  /* populate the array of services */
  for (iter=0; iter<service_group->n_services; iter++)
  {
    dexterplugin_service_copy (service_group->services + iter, registered_services + iter);

#ifdef G_ENABLE_DEBUG
    g_message ("%s: plugin service: \n \
               type:  [%s] \n \
               name:  [%s] \n \
               blurb: [%s] \n \
               uuid:  [%s]",
               G_STRFUNC,
               g_type_name (service_group->services[iter].type),
               service_group->services[iter].name,
               service_group->services[iter].blurb,
               service_group->services[iter].uuid);
#endif
  }

  return service_group;
}
